Package: topdownr
Title: Investigation of Fragmentation Conditions in Top-Down Proteomics
Version: 1.33.0
Authors@R: c(person("Sebastian", "Gibb", role=c("aut", "cre"),
                    email="mail@sebastiangibb.de",
                    comment=c(ORCID="0000-0001-7406-4443")),
             person("Pavel", "Shliaha", role="aut",
                    email="pavels@bmb.sdu.dk",
                    comment=c(ORCID="0000-0003-3092-0724")),
             person(c("Ole", "Nørregaard"), "Jensen", role="aut",
                    email="jenseno@bmb.sdu.dk",
                    comment=c(ORCID="0000-0003-1862-8528")))
Description: The topdownr package allows automatic and systemic investigation
    of fragment conditions. It creates Thermo Orbitrap Fusion Lumos method
    files to test hundreds of fragmentation conditions. Additionally it
    provides functions to analyse and process the generated MS data and
    determine the best conditions to maximise overall fragment coverage.
Depends: R (>= 3.5), methods, BiocGenerics (>= 0.20.0), ProtGenerics
        (>= 1.10.0), Biostrings (>= 2.42.1), S4Vectors (>= 0.12.2)
Imports: grDevices, stats, tools, utils, Biobase, Matrix (>= 1.4-2),
        MSnbase (>= 2.33.5), PSMatch (>= 1.11.4), ggplot2 (>= 2.2.1),
        mzR (>= 2.27.5)
Suggests: topdownrdata (>= 0.2), knitr, rmarkdown, ranger, testthat,
        BiocStyle, xml2
License: GPL (>= 3)
URL: https://codeberg.org/sgibb/topdownr/
BugReports: https://codeberg.org/sgibb/topdownr/issues/
LazyData: true
VignetteBuilder: knitr
Roxygen: list(markdown=TRUE)
RoxygenNote: 7.3.2
biocViews: ImmunoOncology, Infrastructure, Proteomics,
        MassSpectrometry, Coverage
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/topdownr
git_branch: devel
git_last_commit: 3979e4f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:40:54 UTC; biocbuild
Author: Sebastian Gibb [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7406-4443>),
  Pavel Shliaha [aut] (ORCID: <https://orcid.org/0000-0003-3092-0724>),
  Ole Nørregaard Jensen [aut] (ORCID:
    <https://orcid.org/0000-0003-1862-8528>)
Maintainer: Sebastian Gibb <mail@sebastiangibb.de>
