#' Deprecated functions in topdownr
#'
#' These functions are provided for compatibility with older versions
#' of \sQuote{MyPkg} only, and will be defunct at the next release.
#'
#' @details The following functions are deprecated and will be made defunct; use
#' the replacement indicated below:
#' - `defaultMs1Settings`: [expandMs1Conditions()] in combination with
#' [createExperimentsFragmentOptimisation()]
#' - `defaultMs2Settings`: [expandTms2Conditions()] in combination with
#' [createExperimentsFragmentOptimisation()]
#'
#' @name topdownr-deprecated
#' @aliases topdownr-deprecated
#' @keywords package deprecated
NULL
